/*
**　高精度ＡＤコンバータ　ＡＤ７７０５
**　１６ビットΣΔ型、リファレンス電圧２．５Ｖ　差動入力
**  薄膜白金温度センサによる温度計
*/

//　液晶モジュール用ヘッダファイル
#include <LiquidCrystal.h>
//　EEPROM用ヘッダファイル
#include <EEPROM.h>

//　液晶モジュールのピン設定
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// ＡＤ７７０５とのインターフェイス（入出力はArdionoのポートの方向）
int DRDY = 2;  // データレディ（入力）
int CS = 3;    //　チップセレクト（出力）
int DIN = 11;  //　データ（出力）
int DOUT = 12;  //　データ（入力）
int SCLK = 13;   //　シリアルクロック（出力）

//　ａｄ７７０５への１バイト送信（ＳＣＬＫとＤＩＮのみ駆動）
void send_AD7705( unsigned char d ) {
  unsigned char i;
  for( i = 0; i < 8; i ++ ) {  //　８ビット分繰り返し
    digitalWrite( SCLK, LOW ); //　クロック線をＬＯＷレベルにする
    if( d & 0x80 ) digitalWrite( DIN, HIGH ); //　データビットを準備
    else           digitalWrite( DIN, LOW );
    digitalWrite( SCLK, HIGH );  //　クロック線をＨＩＧＨレベルに戻す
    d *= 2;  //　左シフト
  }
}

//　ＡＤ７７０５から１バイト受信（ＳＣＬＫのみ駆動）
unsigned char recv_AD7705() {
  unsigned char i, d;
  d = 0;
  for( i = 0; i < 8; i ++ ) {
    digitalWrite( SCLK, LOW ); //　クロック線をＬＯＷレベルにする
    d *= 2; //左シフト  
    if( HIGH == digitalRead( DOUT ) ) d += 1;  // HIGHならビットを立てる
    digitalWrite( SCLK, HIGH );  //　クロック線をＨＩＧＨレベルに戻す
  }
  return d;
}

//　ＡＤ７７０５の初期化
void setup_AD7705() {
//　１．ポートの向きを設定
  pinMode( DRDY, INPUT );
  pinMode( CS, OUTPUT );
  pinMode( DIN, OUTPUT );
  pinMode( DOUT, INPUT );
  pinMode( SCLK, OUTPUT );
//　２．出力ポートはＨレベルにしておく
  digitalWrite( CS, HIGH );
  digitalWrite( DIN, HIGH );
  digitalWrite( SCLK, HIGH );
// ＡＤ７７０５シリアルポート初期化
  delay( 10 );    // １０ｍＳ待ち
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0xff );    //　最低３２ビットのＨレベルを送信することで
  send_AD7705( 0xff );    //　ＡＤ７７０５のシリアルポートをリセットできる
  send_AD7705( 0xff );
  send_AD7705( 0xff );
  send_AD7705( 0xff );
  digitalWrite( CS, HIGH );  //　リセット終了
  delay( 1 );
// 初期化:クロック設定
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x20 );    //　クロックレジスタを選択
  digitalWrite( CS, HIGH ); 
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x08 );    //　クロック、出力更新レートを選択
  // 電源周波数が６０Ｈｚの地域では０ｘ０８、５０Ｈｚの地域では０ｘ０９
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
// 初期化:セットアップレジスタ設定による自動校正
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x10 );    //　セットアップレジスタを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x40 );    //　バイポーラ動作、セルフキャリブレーション実行
  digitalWrite( CS, HIGH ); 
  delay( 1 );
  while( LOW == digitalRead( DRDY ) ) {  // 校正終了待ち
    delay( 1 );
  }
// 初期化:セットアップレジスタ設定によるゲイン設定
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x10 );    //　セットアップレジスタを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x28 );    //　バイポーラ動作、ゲイン３２倍に切り替え
  digitalWrite( CS, HIGH ); 
}

// ＡＤ７７０５からのデータ読み出し（変換が終わるまで待ってデータを読み出す）
unsigned int conv_AD7705() {
  unsigned int d;
  unsigned char h, l;

  while( LOW == digitalRead( DRDY ) ) {  // 変換終了待ち
    delay( 1 );
  }
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x38 );    //　データレジスタ読み出しを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  h = recv_AD7705();        //　上位バイト
  l = recv_AD7705();        //　下位バイト
  digitalWrite( CS, HIGH );
  d = ( h << 8 ) + l;
  return d;
}

//　温度校正のためのEEPROMデータエリア
int EE_ZERO = 0;
int EE_HUNDRED = 4;
int EE_CALIB = 8;

//　校正データをＲＡＭにコピーするためのエリア
unsigned long int zero, hundred;

//　温度校正ルーチン
void temp_calib() {
  char c;
  unsigned long int d;
  int i;
  
  c = Serial.read();
  if( 'C' != c && 'c' != c ) return;
  Serial.print( zero );
  Serial.print( ' ' );
  Serial.println( hundred );
//　０度Ｃか、１００度Ｃか、データ破棄か
  Serial.print( "Calibrate 0)0deg.C 1)100deg.C X)discard ?" );
  while(  0 == Serial.available() ) { }  //　入力待ち
  c = Serial.read();
//　データ破棄：生データ表示に戻す
  if( 'x' == c || 'X' == c ) {
    Serial.println( "0" );
    for( i = EE_ZERO; i <= EE_CALIB; i ++ ) {
      EEPROM.write( i, 0 ); 
    }
    return;
  }
//　０度Ｃでの校正（かき氷＋水で行う）
  else if( '0' == c ) {
    Serial.println( "0" );
//　１０回の総和をとってＥＥＰＲＯＭに保存
    d = 0;
    for( i = 0; i < 10; i ++ ) d += conv_AD7705();
    zero = d;  // ＲＡＭにもコピー
    EEPROM.write( EE_ZERO, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_ZERO + 1, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_ZERO + 2, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_ZERO + 3, (unsigned char)( 0xff & d ) );
    Serial.println( "0 degree C calibrated." );
  }
//　１００度Ｃでの校正（沸騰している水で行う）
  else if( '1' == c ) {
    Serial.println( "1" );
//　１０回の総和をとってＥＥＰＲＯＭに保存
    d = 0;
    for( i = 0; i < 10; i ++ ) d += conv_AD7705();
    hundred = d;  // ＲＡＭにもコピー
    EEPROM.write( EE_HUNDRED, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_HUNDRED + 1, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_HUNDRED + 2, (unsigned char)( 0xff & d ) );
    d >>= 8;
    EEPROM.write( EE_HUNDRED + 3, (unsigned char)( 0xff & d ) );
    Serial.println( "100 degree C calibrated." );
  }
  else return;  //　それ以外の文字だったら校正しない
//　０度Ｃ、１００度Ｃ両方校正できていたら校正済のフラグをセット
  if( ( 0 != EEPROM.read( EE_ZERO ) | EEPROM.read( EE_ZERO + 1 ) |
             EEPROM.read( EE_ZERO + 2 ) | EEPROM.read( EE_ZERO + 3 ) ) &&
      ( 0 != EEPROM.read( EE_HUNDRED ) | EEPROM.read( EE_HUNDRED + 1 ) |
             EEPROM.read( EE_HUNDRED + 2 ) | EEPROM.read( EE_HUNDRED + 3 ) ) ) {
    EEPROM.write( EE_CALIB, 'C' );
  }
}

//　初期化のために最初に１回だけ呼ばれる関数
void setup() {
  int i;

  lcd.begin(16, 2);        //　液晶モジュールの初期化
  Serial.begin( 9600 );   //　シリアルポートの初期化
  setup_AD7705();          //　ＡＤ７７０５の初期化
//　ＥＥＰＲＯＭから校正データをＲＡＭに準備
  if( 'C' == EEPROM.read( EE_CALIB ) ) {
    zero = (long)EEPROM.read( EE_ZERO ) + ( (long)EEPROM.read( EE_ZERO + 1 ) * 256L ) +
         ( (long)EEPROM.read( EE_ZERO + 2 ) * 65536L );
    hundred = (long)EEPROM.read( EE_HUNDRED ) + ( (long)EEPROM.read( EE_HUNDRED + 1 ) *256L ) +
            ( (long)EEPROM.read( EE_HUNDRED + 2 ) * 65536L );
  }
  else {
    zero = 0;
    hundred = 0;
  }
/*  for( i = 0; i < 10; i ++ ) {
    Serial.print( EEPROM.read( i ) );
    Serial.print( ' ' );
  }
  for( ; ; ); 
*/
}

//　動作時に繰り返し呼ばれる関数
void loop() {
  long int d;
  int i;
  float t;

//　シリアルポートからなにか入ってきていたら校正ルーチンを実行
  if( 0 != Serial.available() ) temp_calib();
//　校正ができていれば温度表示、できていなければ生データ表示
  if( 'C' == EEPROM.read( EE_CALIB ) ) {
//　校正ができている場合
    d = 0;
    for( i = 0; i < 10; i ++ ) d += conv_AD7705();
    d -= zero;
    t = ( float )d / ( float )( hundred - zero ) * 100.0;
    lcd.setCursor(0,0);
    lcd.print( t, 3 );
    lcd.print( ' ' );
    Serial.println( t, 3 );
    delay( 100 );
  }
  else {
//　校正ができていない場合
    d = 0;
    for( i = 0; i < 10; i ++ ) d += conv_AD7705();
    lcd.setCursor(0,0);
    lcd.print( "R:" );    //　生データであることの表示
    lcd.print( d );
    lcd.print( ' ' );
    Serial.println( d );
    delay( 100 );
  }
}

